@@ -33,6 +33,7 @@ from pycbc.distributions.uniform_log import UniformLog10
 from pycbc.distributions.spins import IndependentChiPChiEff
 from pycbc.distributions.qnm import UniformF0Tau
 from pycbc.distributions.joint import JointDistribution
+from pycbc.distributions.external import External
 
 # a dict of all available distributions
 distribs = {
@@ -50,6 +51,7 @@ distribs = {
     UniformSky.name : UniformSky,
     UniformLog10.name : UniformLog10,
     UniformF0Tau.name : UniformF0Tau,
+    External.name: External
 }
 
 def read_distributions_from_config(cp, section="prior"):
