@@ -812,11 +812,9 @@ def compute_u_val_for_sky_loc_stat_no_phase(hplus, hcross, hphccorr,
     # Initialize tan_kappa array
     u_val = denom * 0.
     # Catch the denominator -> 0 case
-    bad_lgc = (denom == 0)
-    u_val[bad_lgc] = 1E17
+    numpy.putmask(u_val, denom == 0, 1E17)
     # Otherwise do normal statistic
-    u_val[~bad_lgc] = (-rhoplusre+overlap*rhocrossre) / \
-        (-rhocrossre+overlap*rhoplusre)
+    numpy.putmask(u_val, denom != 0, (-rhoplusre+overlap*rhocrossre)/(-rhocrossre+overlap*rhoplusre))
     coa_phase = numpy.zeros(len(indices), dtype=numpy.float32)
 
     return u_val, coa_phase
@@ -1682,6 +1680,10 @@ class LiveBatchMatchedFilter(object):
         # Find the peaks in our SNR times series from the various templates
         i = 0
         for htilde in tgroup:
+            if hasattr(htilde, 'time_offset'):
+                if 'time_offset' not in result:
+                    result['time_offset'] = []
+
             l = htilde.out[seg].abs_arg_max()
 
             sgm = htilde.sigmasq(psd)
@@ -1716,8 +1718,11 @@ class LiveBatchMatchedFilter(object):
 
             for key in tkeys:
                 result[key].append(htilde.dict_params[key])
-            i += 1
 
+            if hasattr(htilde, 'time_offset'):
+                result['time_offset'].append(htilde.time_offset)
+
+            i += 1
 
         result['snr'] = abs(snr[0:i])
         result['coa_phase'] = numpy.angle(snr[0:i])
@@ -1728,6 +1733,9 @@ class LiveBatchMatchedFilter(object):
         for key in tkeys:
             result[key] = numpy.array(result[key])
 
+        if 'time_offset' in result:
+            result['time_offset'] = numpy.array(result['time_offset'])
+
         return result, veto_info
 
 def followup_event_significance(ifo, data_reader, bank,
