@@ -23,10 +23,13 @@ from pickle import UnpicklingError
 from epsie import load_state
 
 from .base_sampler import BaseSamplerFile
-from .base_multitemper import (MultiTemperedMCMCIO, MultiTemperedMetadataIO)
+from .base_mcmc import MCMCMetadataIO
+from .base_multitemper import (CommonMultiTemperedMetadataIO,
+                               write_samples,
+                               read_raw_samples)
 
 
-class EpsieFile(MultiTemperedMCMCIO, MultiTemperedMetadataIO,
+class EpsieFile(MCMCMetadataIO, CommonMultiTemperedMetadataIO,
                 BaseSamplerFile):
     """Class to handle IO for Epsie's parallel-tempered sampler."""
 
@@ -57,10 +60,7 @@ class EpsieFile(MultiTemperedMCMCIO, MultiTemperedMetadataIO,
         """
         super(EpsieFile, self).write_sampler_metadata(sampler)
         self[self.sampler_group].attrs['seed'] = sampler.seed
-        try:
-            self[self.sampler_group]["betas"][:] = sampler.betas
-        except KeyError:
-            self[self.sampler_group]["betas"] = sampler.betas
+        self.write_data("betas", sampler.betas, path=self.sampler_group)
 
     def thin(self, thin_interval):
         """Thins the samples on disk to the given thinning interval.
@@ -68,10 +68,14 @@ class EpsieFile(MultiTemperedMCMCIO, MultiTemperedMetadataIO,
         Also thins the acceptance ratio and the temperature data, both of
         which are stored in the ``sampler_info`` group.
         """
-        # thin the samples
+        # We'll need to know what the new interval to thin by will be
+        # so we can properly thin the acceptance ratio and temperatures swaps.
+        # We need to do this before calling the base thin, as we need to know
+        # what the current thinned by is.
+        new_interval = thin_interval // self.thinned_by
+        # now thin the samples
         super(EpsieFile, self).thin(thin_interval)
         # thin the acceptance ratio
-        new_interval = thin_interval // self.thinned_by
         self._thin_data(self.sampler_group, ['acceptance_ratio'],
                         new_interval)
         # thin the temperature swaps; since these may not happen every
@@ -79,8 +83,48 @@ class EpsieFile(MultiTemperedMCMCIO, MultiTemperedMetadataIO,
         ts_group = '/'.join([self.sampler_group, 'temperature_swaps'])
         ts_thin_interval = new_interval // self.swap_interval
         if ts_thin_interval > 1:
-            self._thin_data(ts_group, ['swap_index', 'acceptance_ratio'],
+            self._thin_data(ts_group, ['swap_index'],
                             ts_thin_interval)
+            self._thin_data(ts_group, ['acceptance_ratio'],
+                            ts_thin_interval)
+
+    def write_samples(self, samples, **kwargs):
+        r"""Writes samples to the given file.
+
+        Calls :py:func:`base_multitemper.write_samples`. See that function for
+        details.
+
+        Parameters
+        ----------
+        samples : dict
+            The samples to write. Each array in the dictionary should have
+            shape ntemps x nwalkers x niterations.
+        \**kwargs :
+            All other keyword arguments are passed to
+            :py:func:`base_multitemper.write_samples`.
+        """
+        write_samples(self, samples, **kwargs)
+
+    def read_raw_samples(self, fields, **kwargs):
+        r"""Base function for reading samples.
+
+        Calls :py:func:`base_multitemper.read_raw_samples`. See that
+        function for details.
+
+        Parameters
+        -----------
+        fields : list
+            The list of field names to retrieve.
+        \**kwargs :
+            All other keyword arguments are passed to
+            :py:func:`base_multitemper.read_raw_samples`.
+
+        Returns
+        -------
+        dict
+            A dictionary of field name -> numpy array pairs.
+        """
+        return read_raw_samples(self, fields, **kwargs)
 
     def write_acceptance_ratio(self, acceptance_ratio, last_iteration=None):
         """Writes the acceptance ratios to the sampler info group.
@@ -213,3 +257,16 @@ class EpsieFile(MultiTemperedMCMCIO, MultiTemperedMetadataIO,
                 # corrupted for some reason
                 valid = False
         return valid
+
+    @staticmethod
+    def _get_optional_args(args, opts, err_on_missing=False, **kwargs):
+        # need this to make sure options called "walkers" are renamed to
+        # "chains"
+        parsed = BaseSamplerFile._get_optional_args(
+            args, opts, err_on_missing=err_on_missing, **kwargs)
+        try:
+            chains = parsed.pop('walkers')
+            parsed['chains'] = chains
+        except KeyError:
+            pass
+        return parsed
