@@ -79,14 +79,10 @@ class EmceeEnsembleSampler(EnsembleSupport, BaseMCMC, BaseSampler):
             logpost_function = 'logposterior'
         model_call = models.CallModel(model, logpost_function)
 
-        # Set up the pool
-        if nprocesses > 1:
-            # these are used to help paralleize over multiple cores / MPI
-            models._global_instance = model_call
-            model_call = models._call_global_model
+        # these are used to help paralleize over multiple cores / MPI
+        models._global_instance = model_call
+        model_call = models._call_global_model
         pool = choose_pool(mpi=use_mpi, processes=nprocesses)
-        if pool is not None:
-            pool.count = nprocesses
 
         # set up emcee
         self.nwalkers = nwalkers
